/***********************************************************************************
	sang_status_defines.h

	Completion status definitions of operations.
************************************************************************************/

#ifndef _SANG_STATUS_DEFINES_H_
#define _SANG_STATUS_DEFINES_H_

#ifndef __WINDOWS__
# define __WINDOWS__
#endif

typedef enum SANG_STATUS{

	//An operation completed successfully
	SANG_STATUS_SUCCESS=0,
	//Two threads attempting to access device at the same time.
	SANG_STATUS_COMMAND_ALREADY_RUNNING,
	//Buffer passed to API is too small.
	SANG_STATUS_BUFFER_TOO_SMALL,
	//Kernel error
	SANG_STATUS_FAILED_TO_LOCK_USER_MEMORY,
	//Kernel error
	SANG_STATUS_FAILED_ALLOCATE_MEMORY,
	//Command is invalid for device type.
	SANG_STATUS_INVALID_DEVICE_REQUEST,
	//General 
	SANG_STATUS_INVALID_PARAMETER,

	//For internal API use.
	SANG_STATUS_DATA_QUEUE_EMPTY,
	SANG_STATUS_DATA_QUEUE_FULL,

	//
	//Return codes specific for data reception:
	//
	//No data was received. NOT an error.
	SANG_STATUS_RX_DATA_TIMEOUT,
	//Data was received.
	SANG_STATUS_RX_DATA_AVAILABLE,

	//
	//Return codes specific for data transmission:
	//
	//Transmit command timed out.
	SANG_STATUS_TX_TIMEOUT,
	//Longer than MTU
	SANG_STATUS_TX_DATA_TOO_LONG,
	//Shorter than minimum
	SANG_STATUS_TX_DATA_TOO_SHORT,
	//For internal API use.
	SANG_STATUS_WAIT_FOR_TX_INTERRUPT,

	//Physical line (T1/E1/Serial) is disconnected.
	SANG_STATUS_LINE_DISCONNECTED,

	//The Communication Protocol (CHDLC/Frame Relay/PPP...) is disconnected.
	SANG_STATUS_PROTOCOL_DISCONNECTED,

	SANG_STATUS_GENERAL_ERROR,
	SANG_STATUS_DEVICE_BUSY,
	SANG_STATUS_INVALID_DEVICE,
	SANG_STATUS_IO_ERROR,
	SANG_STATUS_UNSUPPORTED_FUNCTION,
	SANG_STATUS_UNSUPPORTED_PROTOCOL,
	SANG_STATUS_DEVICE_ALREADY_EXIST,
	SANG_STATUS_DEV_INIT_INCOMPLETE,
	SANG_STATUS_TRACE_QUEUE_EMPTY,
	SANG_STATUS_OPTION_NOT_SUPPORTED,

	//TDM Voice API definitions
	SANG_STATUS_TDM_EVENT_AVAILABLE
};


#define SDLA_DECODE_SANG_STATUS(status)	\
(status == SANG_STATUS_SUCCESS) ? "SANG_STATUS_SUCCESS" :\
(status == SANG_STATUS_COMMAND_ALREADY_RUNNING) ? "SANG_STATUS_COMMAND_ALREADY_RUNNING":\
(status == SANG_STATUS_BUFFER_TOO_SMALL) ? "SANG_STATUS_BUFFER_TOO_SMALL":\
(status == SANG_STATUS_FAILED_TO_LOCK_USER_MEMORY) ? "SANG_STATUS_FAILED_TO_LOCK_USER_MEMORY":\
(status == SANG_STATUS_FAILED_ALLOCATE_MEMORY) ? "SANG_STATUS_FAILED_ALLOCATE_MEMORY":\
(status == SANG_STATUS_INVALID_DEVICE_REQUEST) ? "SANG_STATUS_INVALID_DEVICE_REQUEST":\
(status == SANG_STATUS_INVALID_PARAMETER) ? "SANG_STATUS_INVALID_PARAMETER":\
(status == SANG_STATUS_DATA_QUEUE_EMPTY) ? "SANG_STATUS_DATA_QUEUE_EMPTY":\
(status == SANG_STATUS_DATA_QUEUE_FULL) ? "SANG_STATUS_DATA_QUEUE_FULL":\
(status == SANG_STATUS_RX_DATA_TIMEOUT) ? "SANG_STATUS_RX_DATA_TIMEOUT":\
(status == SANG_STATUS_RX_DATA_AVAILABLE) ? "SANG_STATUS_RX_DATA_AVAILABLE":\
(status == SANG_STATUS_TX_TIMEOUT) ? "SANG_STATUS_TX_TIMEOUT":\
(status == SANG_STATUS_TX_DATA_TOO_LONG) ? "SANG_STATUS_TX_DATA_TOO_LONG":\
(status == SANG_STATUS_TX_DATA_TOO_SHORT) ? "SANG_STATUS_TX_DATA_TOO_SHORT":\
(status == SANG_STATUS_WAIT_FOR_TX_INTERRUPT) ? "SANG_STATUS_WAIT_FOR_TX_INTERRUPT":\
(status == SANG_STATUS_LINE_DISCONNECTED) ? "SANG_STATUS_LINE_DISCONNECTED":\
(status == SANG_STATUS_PROTOCOL_DISCONNECTED) ? "SANG_STATUS_PROTOCOL_DISCONNECTED":\
(status == SANG_STATUS_GENERAL_ERROR) ? "SANG_STATUS_GENERAL_ERROR":\
(status == SANG_STATUS_DEVICE_BUSY) ? "SANG_STATUS_DEVICE_BUSY":\
(status == SANG_STATUS_INVALID_DEVICE) ? "SANG_STATUS_INVALID_DEVICE":\
(status == SANG_STATUS_IO_ERROR) ? "SANG_STATUS_IO_ERROR":\
(status == SANG_STATUS_UNSUPPORTED_FUNCTION) ? "SANG_STATUS_UNSUPPORTED_FUNCTION":\
(status == SANG_STATUS_UNSUPPORTED_PROTOCOL) ? "SANG_STATUS_UNSUPPORTED_PROTOCOL":\
(status == SANG_STATUS_DEVICE_ALREADY_EXIST) ? "SANG_STATUS_DEVICE_ALREADY_EXIST":\
(status == SANG_STATUS_DEV_INIT_INCOMPLETE) ? "SANG_STATUS_DEV_INIT_INCOMPLETE":\
"Status Unknown"

#endif //_SANG_STATUS_DEFINES_H_
